@@ -20,47 +20,56 @@
  * 
  */
 /**
-    File Name:          12.6.3-2.js
-    ECMA Section:       12.6.3 The for...in Statement
-    Description:        Check the Boolean Object
+    File Name:          12.6.2-1.js
+    ECMA Section:       12.6.2 The for Statement
+
+                        1. first expression is not present.
+                        2. second expression is not present
+                        3. third expression is not present
 
 
     Author:             christine@netscape.com
-    Date:               11 september 1997
+    Date:               15 september 1997
 */
 
-    var SECTION = "12.6.3-2";
+    var SECTION = "12.6.2-1";
     var VERSION = "ECMA_1";
     startTest();
-    var TITLE   = "The for..in statment";
+    var TITLE   = "The for statment";
 
     writeHeaderToLog( SECTION + " "+ TITLE);
 
-    var testcases = getTestCases();
+    var testcases = new getTestCases();
 
     test();
 
-function test() {
-    for ( tc=0; tc < testcases.length; tc++ ) {
-        testcases[tc].passed = writeTestCaseResult(
-                            testcases[tc].expect,
-                            testcases[tc].actual,
-                            testcases[tc].description +" = "+
-                            testcases[tc].actual );
-
-        testcases[tc].reason += ( testcases[tc].passed ) ? "" : "wrong value ";
-    }
-    stopTest();
-    return ( testcases );
-}
 function getTestCases() {
     var array = new Array();
-    var item = 0;
-
-    array[item++] = new TestCase(   SECTION,
-                                    "Boolean.prototype.foo = 34; for ( j in Boolean ) Boolean[j]",
-                                    34,
-                                    eval("Boolean.prototype.foo = 34; for ( j in Boolean ) Boolean[j] ") );
-
+    array[0] = new TestCase( "12.6.2-1", "for statement",  99,     "" );
     return ( array );
 }
+
+function testprogram() {
+    myVar = 0;
+
+    for ( ; ; ) {
+        if ( ++myVar == 99 )
+            break;
+    }
+
+    return myVar;
+}
+function test() {
+        testcases[0].actual = testprogram();
+
+        testcases[0].passed = writeTestCaseResult(
+                    testcases[0].expect,
+                    testcases[0].actual,
+                    testcases[0].description +" = "+ testcases[0].actual );
+
+        testcases[0].reason += ( testcases[0].passed ) ? "" : "wrong value ";
+
+        stopTest();
+
+        return ( testcases );
+}
