@@ -1,6 +1,7 @@
 /*
  * Copyright (C) 2007, 2008, 2010 Apple Inc.  All rights reserved.
  * Copyright (C) 2009 Joseph Pecoraro
+ * Copyright (C) 2013 Samsung Electronics. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -114,14 +115,10 @@ WebInspector.ResourcesPanel = function(database)
 
     WebInspector.databaseModel.databases().forEach(this._addDatabase.bind(this));
     WebInspector.databaseModel.addEventListener(WebInspector.DatabaseModel.Events.DatabaseAdded, this._databaseAdded, this);
-
-    WebInspector.domStorageModel.storages().forEach(this._addDOMStorage.bind(this));
-    WebInspector.domStorageModel.addEventListener(WebInspector.DOMStorageModel.Events.DOMStorageAdded, this._domStorageAdded, this);
-    WebInspector.domStorageModel.addEventListener(WebInspector.DOMStorageModel.Events.DOMStorageUpdated, this._domStorageUpdated, this);
 }
 
 WebInspector.ResourcesPanel.prototype = {
-    get statusBarItems()
+    statusBarItems: function()
     {
         return [this.storageViewStatusBarItemsContainer];
     },
@@ -136,6 +133,7 @@ WebInspector.ResourcesPanel.prototype = {
     {
         if (!this._initialized && this.isShowing() && this._cachedResourcesWereLoaded) {
             this._populateResourceTree();
+            this._populateDOMStorageTree();
             this._populateApplicationCacheTree();
             this._initDefaultSelection();
             this._initialized = true;
@@ -193,7 +191,7 @@ WebInspector.ResourcesPanel.prototype = {
         this.sessionStorageListTreeElement.removeChildren();
         this.cookieListTreeElement.removeChildren();
 
-        if (this.visibleView)
+        if (this.visibleView && !(this.visibleView instanceof WebInspector.StorageCategoryView))
             this.visibleView.detach();
 
         this.storageViewStatusBarItemsContainer.removeChildren();
@@ -331,7 +329,7 @@ WebInspector.ResourcesPanel.prototype = {
      */
     _domStorageAdded: function(event)
     {
-        var domStorage = /** @type {WebInspector.DOMStorage}*/ (event.data);
+        var domStorage = /** @type {WebInspector.DOMStorage} */ (event.data);
         this._addDOMStorage(domStorage);
     },
 
@@ -340,6 +338,8 @@ WebInspector.ResourcesPanel.prototype = {
      */
     _addDOMStorage: function(domStorage)
     {
+        console.assert(!this._domStorageTreeElements.get(domStorage));
+
         var domStorageTreeElement = new WebInspector.DOMStorageTreeElement(this, domStorage, (domStorage.isLocalStorage ? "local-storage" : "session-storage"));
         this._domStorageTreeElements.put(domStorage, domStorageTreeElement);
         if (domStorage.isLocalStorage)
@@ -348,6 +348,32 @@ WebInspector.ResourcesPanel.prototype = {
             this.sessionStorageListTreeElement.appendChild(domStorageTreeElement);
     },
 
+    /**
+     * @param {WebInspector.Event} event
+     */
+    _domStorageRemoved: function(event)
+    {
+        var domStorage = /** @type {WebInspector.DOMStorage} */ (event.data);
+        this._removeDOMStorage(domStorage);
+    },
+
+    /**
+     * @param {WebInspector.DOMStorage} domStorage
+     */
+    _removeDOMStorage: function(domStorage)
+    {
+        var treeElement = this._domStorageTreeElements.get(domStorage);
+        if (!treeElement)
+            return;
+        var wasSelected = treeElement.selected;
+        var parentListTreeElement = treeElement.parent;
+        parentListTreeElement.removeChild(treeElement);
+        if (wasSelected)
+            parentListTreeElement.select();
+        this._domStorageTreeElements.remove(treeElement);
+        this._domStorageViews.remove(domStorage);
+    },
+
     /**
      * @param {WebInspector.Database} database
      */
@@ -469,7 +495,7 @@ WebInspector.ResourcesPanel.prototype = {
         var view;
         view = this._domStorageViews.get(domStorage);
         if (!view) {
-            view = new WebInspector.DOMStorageItemsView(domStorage);
+            view = new WebInspector.DOMStorageItemsView(domStorage, WebInspector.domStorageModel);
             this._domStorageViews.put(domStorage, view);
         }
 
@@ -487,6 +513,14 @@ WebInspector.ResourcesPanel.prototype = {
         this._innerShowView(view);
     },
 
+    /**
+     * @param {string} cookieDomain
+     */
+    clearCookies: function(cookieDomain)
+    {
+        this._cookieViews[cookieDomain].clear();
+    },
+
     showApplicationCache: function(frameId)
     {
         if (!this._applicationCacheViews[frameId])
@@ -523,7 +557,7 @@ WebInspector.ResourcesPanel.prototype = {
         this.visibleView = view;
 
         this.storageViewStatusBarItemsContainer.removeChildren();
-        var statusBarItems = view.statusBarItems || [];
+        var statusBarItems = view.statusBarItems() || [];
         for (var i = 0; i < statusBarItems.length; ++i)
             this.storageViewStatusBarItemsContainer.appendChild(statusBarItems[i]);
     },
@@ -572,15 +606,11 @@ WebInspector.ResourcesPanel.prototype = {
         database.getTableNames(tableNamesCallback);
     },
 
-    /**
-     * @param {WebInspector.Event} event
-     */
-    _domStorageUpdated: function(event)
+    _populateDOMStorageTree: function()
     {
-        var storage = /** @type {WebInspector.DOMStorage}*/ (event.data);
-        var view = this._domStorageViews.get(storage);
-        if (this.visibleView && view === this.visibleView)
-            view.update();
+        WebInspector.domStorageModel.storages().forEach(this._addDOMStorage.bind(this));
+        WebInspector.domStorageModel.addEventListener(WebInspector.DOMStorageModel.Events.DOMStorageAdded, this._domStorageAdded, this);
+        WebInspector.domStorageModel.addEventListener(WebInspector.DOMStorageModel.Events.DOMStorageRemoved, this._domStorageRemoved, this);
     },
 
     _populateApplicationCacheTree: function()
@@ -944,8 +974,10 @@ WebInspector.BaseStorageTreeElement.prototype = {
         }
     },
 
-    onselect: function()
+    onselect: function(selectedByUser)
     {
+        if (!selectedByUser)
+            return;
         var itemURL = this.itemURL;
         if (itemURL)
             WebInspector.settings.resourcesLastSelectedItem.set(itemURL);
@@ -1006,9 +1038,9 @@ WebInspector.StorageCategoryTreeElement.prototype = {
         return "category://" + this._categoryName;
     },
 
-    onselect: function()
+    onselect: function(selectedByUser)
     {
-        WebInspector.BaseStorageTreeElement.prototype.onselect.call(this);
+        WebInspector.BaseStorageTreeElement.prototype.onselect.call(this, selectedByUser);
         this._storagePanel.showCategoryView(this._categoryName);
     },
 
@@ -1063,9 +1095,9 @@ WebInspector.FrameTreeElement.prototype = {
         return "frame://" + encodeURI(this.displayName);
     },
 
-    onselect: function()
+    onselect: function(selectedByUser)
     {
-        WebInspector.BaseStorageTreeElement.prototype.onselect.call(this);
+        WebInspector.BaseStorageTreeElement.prototype.onselect.call(this, selectedByUser);
         this._storagePanel.showCategoryView(this.displayName);
 
         this.listItemElement.removeStyleClass("hovered");
@@ -1171,9 +1203,9 @@ WebInspector.FrameResourceTreeElement.prototype = {
         return this._resource.url;
     },
 
-    onselect: function()
+    onselect: function(selectedByUser)
     {
-        WebInspector.BaseStorageTreeElement.prototype.onselect.call(this);
+        WebInspector.BaseStorageTreeElement.prototype.onselect.call(this, selectedByUser);
         this._storagePanel._showResourceView(this._resource);
     },
 
@@ -1338,9 +1370,9 @@ WebInspector.DatabaseTreeElement.prototype = {
         return "database://" + encodeURI(this._database.name);
     },
 
-    onselect: function()
+    onselect: function(selectedByUser)
     {
-        WebInspector.BaseStorageTreeElement.prototype.onselect.call(this);
+        WebInspector.BaseStorageTreeElement.prototype.onselect.call(this, selectedByUser);
         this._storagePanel._showDatabase(this._database);
     },
 
@@ -1382,9 +1414,9 @@ WebInspector.DatabaseTableTreeElement.prototype = {
         return "database://" + encodeURI(this._database.name) + "/" + encodeURI(this._tableName);
     },
 
-    onselect: function()
+    onselect: function(selectedByUser)
     {
-        WebInspector.BaseStorageTreeElement.prototype.onselect.call(this);
+        WebInspector.BaseStorageTreeElement.prototype.onselect.call(this, selectedByUser);
         this._storagePanel._showDatabase(this._database, this._tableName);
     },
 
@@ -1532,7 +1564,7 @@ WebInspector.FileSystemListTreeElement.prototype = {
     _handleContextMenuEvent: function(event)
     {
         var contextMenu = new WebInspector.ContextMenu(event);
-        contextMenu.appendItem(WebInspector.UIString("Refresh FileSystem List"), this._refreshFileSystem.bind(this));
+        contextMenu.appendItem(WebInspector.UIString(WebInspector.useLowerCaseMenuTitles() ? "Refresh FileSystem list" : "Refresh FileSystem List"), this._refreshFileSystem.bind(this));
         contextMenu.show();
     },
 
@@ -1611,7 +1643,7 @@ WebInspector.IDBDatabaseTreeElement.prototype = {
         contextMenu.show();
     },
 
-    _refreshIndexedDB: function(event)
+    _refreshIndexedDB: function()
     {
         this._model.refreshDatabaseNames();
     },
@@ -1654,9 +1686,9 @@ WebInspector.IDBDatabaseTreeElement.prototype = {
         this.tooltip = WebInspector.UIString("Version") + ": " + this._database.version;
     },
 
-    onselect: function()
+    onselect: function(selectedByUser)
     {
-        WebInspector.BaseStorageTreeElement.prototype.onselect.call(this);
+        WebInspector.BaseStorageTreeElement.prototype.onselect.call(this, selectedByUser);
         if (!this._view)
             this._view = new WebInspector.IDBDatabaseView(this._database);
 
@@ -1705,6 +1737,27 @@ WebInspector.IDBObjectStoreTreeElement.prototype = {
         return "indexedDB://" + this._databaseId.securityOrigin + "/" + this._databaseId.name + "/" + this._objectStore.name;
     },
 
+    onattach: function()
+    {
+        WebInspector.BaseStorageTreeElement.prototype.onattach.call(this);
+        this.listItemElement.addEventListener("contextmenu", this._handleContextMenuEvent.bind(this), true);
+    },
+
+    _handleContextMenuEvent: function(event)
+    {
+        var contextMenu = new WebInspector.ContextMenu(event);
+        contextMenu.appendItem(WebInspector.UIString("Clear"), this._clearObjectStore.bind(this));
+        contextMenu.show();
+    },
+
+    _clearObjectStore: function()
+    {
+        function callback() {
+            this.update(this._objectStore);
+        }
+        this._model.clearObjectStore(this._databaseId, this._objectStore.name, callback.bind(this));
+    },
+
    /**
      * @param {WebInspector.IndexedDBModel.ObjectStore} objectStore
      */
@@ -1755,9 +1808,9 @@ WebInspector.IDBObjectStoreTreeElement.prototype = {
         this.tooltip = tooltipString
     },
 
-    onselect: function()
+    onselect: function(selectedByUser)
     {
-        WebInspector.BaseStorageTreeElement.prototype.onselect.call(this);
+        WebInspector.BaseStorageTreeElement.prototype.onselect.call(this, selectedByUser);
         if (!this._view)
             this._view = new WebInspector.IDBDataView(this._model, this._databaseId, this._objectStore, null);
 
@@ -1835,9 +1888,9 @@ WebInspector.IDBIndexTreeElement.prototype = {
         this.tooltip = tooltipLines.join("\n");
     },
 
-    onselect: function()
+    onselect: function(selectedByUser)
     {
-        WebInspector.BaseStorageTreeElement.prototype.onselect.call(this);
+        WebInspector.BaseStorageTreeElement.prototype.onselect.call(this, selectedByUser);
         if (!this._view)
             this._view = new WebInspector.IDBDataView(this._model, this._databaseId, this._objectStore, this._index);
 
@@ -1859,19 +1912,19 @@ WebInspector.IDBIndexTreeElement.prototype = {
  */
 WebInspector.DOMStorageTreeElement = function(storagePanel, domStorage, className)
 {
-    WebInspector.BaseStorageTreeElement.call(this, storagePanel, null, domStorage.domain ? domStorage.domain : WebInspector.UIString("Local Files"), ["domstorage-storage-tree-item", className]);
+    WebInspector.BaseStorageTreeElement.call(this, storagePanel, null, domStorage.securityOrigin ? domStorage.securityOrigin : WebInspector.UIString("Local Files"), ["domstorage-storage-tree-item", className]);
     this._domStorage = domStorage;
 }
 
 WebInspector.DOMStorageTreeElement.prototype = {
     get itemURL()
     {
-        return "storage://" + this._domStorage.domain + "/" + (this._domStorage.isLocalStorage ? "local" : "session");
+        return "storage://" + this._domStorage.securityOrigin + "/" + (this._domStorage.isLocalStorage ? "local" : "session");
     },
 
-    onselect: function()
+    onselect: function(selectedByUser)
     {
-        WebInspector.BaseStorageTreeElement.prototype.onselect.call(this);
+        WebInspector.BaseStorageTreeElement.prototype.onselect.call(this, selectedByUser);
         this._storagePanel._showDOMStorage(this._domStorage);
     },
 
@@ -1894,9 +1947,33 @@ WebInspector.CookieTreeElement.prototype = {
         return "cookies://" + this._cookieDomain;
     },
 
-    onselect: function()
+    onattach: function()
     {
-        WebInspector.BaseStorageTreeElement.prototype.onselect.call(this);
+        WebInspector.BaseStorageTreeElement.prototype.onattach.call(this);
+        this.listItemElement.addEventListener("contextmenu", this._handleContextMenuEvent.bind(this), true);
+    },
+
+    /**
+     * @param {Event} event
+     */
+    _handleContextMenuEvent: function(event)
+    {
+        var contextMenu = new WebInspector.ContextMenu(event);
+        contextMenu.appendItem(WebInspector.UIString("Clear"), this._clearCookies.bind(this));
+        contextMenu.show();
+    },
+
+    /**
+     * @param {string} domain
+     */
+    _clearCookies: function(domain)
+    {
+        this._storagePanel.clearCookies(this._cookieDomain);
+    },
+
+    onselect: function(selectedByUser)
+    {
+        WebInspector.BaseStorageTreeElement.prototype.onselect.call(this, selectedByUser);
         this._storagePanel.showCookies(this, this._cookieDomain);
     },
 
@@ -1926,9 +2003,9 @@ WebInspector.ApplicationCacheManifestTreeElement.prototype = {
         return this._manifestURL;
     },
 
-    onselect: function()
+    onselect: function(selectedByUser)
     {
-        WebInspector.BaseStorageTreeElement.prototype.onselect.call(this);
+        WebInspector.BaseStorageTreeElement.prototype.onselect.call(this, selectedByUser);
         this._storagePanel.showCategoryView(this._manifestURL);
     },
 
@@ -1979,9 +2056,9 @@ WebInspector.ApplicationCacheFrameTreeElement.prototype = {
         this._refreshTitles();
     },
 
-    onselect: function()
+    onselect: function(selectedByUser)
     {
-        WebInspector.BaseStorageTreeElement.prototype.onselect.call(this);
+        WebInspector.BaseStorageTreeElement.prototype.onselect.call(this, selectedByUser);
         this._storagePanel.showApplicationCache(this._frameId);
     },
 
@@ -2012,16 +2089,16 @@ WebInspector.FileSystemTreeElement.prototype = {
         return "filesystem://" + this._fileSystem.name;
     },
 
-    onselect: function()
+    onselect: function(selectedByUser)
     {
-        WebInspector.BaseStorageTreeElement.prototype.onselect.call(this);
+        WebInspector.BaseStorageTreeElement.prototype.onselect.call(this, selectedByUser);
         this._fileSystemView = new WebInspector.FileSystemView(this._fileSystem);
         this._storagePanel.showFileSystem(this._fileSystemView);
     },
 
     clear: function()
     {
-        if (this.fileSystemView && this._storagePanel.visibleView == this.fileSystemView)
+        if (this.fileSystemView && this._storagePanel.visibleView === this.fileSystemView)
             this._storagePanel.closeVisibleView();
     },
 
@@ -2079,7 +2156,7 @@ WebInspector.ResourcesSearchController.prototype = {
         if (this._lastTreeElement !== currentTreeElement || this._lastIndex === -1)
             return this._searchResult(currentTreeElement, 0);
 
-        if (this._lastIndex == currentTreeElement.searchMatchesCount - 1)
+        if (this._lastIndex === currentTreeElement.searchMatchesCount - 1)
             return this._searchResult(this._traverser.next(currentTreeElement), 0, this._currentMatchIndex % this._matchesCount + 1);
 
         return this._searchResult(currentTreeElement, this._lastIndex + 1, this._currentMatchIndex + 1);
