@@ -26,29 +26,31 @@
 #ifndef GraphicsContext3D_h
 #define GraphicsContext3D_h
 
-#include "IntRect.h"
 #include "GraphicsTypes3D.h"
+#include "Image.h"
+#include "IntRect.h"
 #include "PlatformLayer.h"
 #include <wtf/HashMap.h>
 #include <wtf/ListHashSet.h>
 #include <wtf/Noncopyable.h>
+#include <wtf/OwnArrayPtr.h>
+#include <wtf/PassOwnArrayPtr.h>
 #include <wtf/RefCounted.h>
 #include <wtf/text/WTFString.h>
 
 // FIXME: Find a better way to avoid the name confliction for NO_ERROR.
-#if ((PLATFORM(CHROMIUM) && OS(WINDOWS)) || PLATFORM(WIN) || (PLATFORM(QT) && OS(WINDOWS)))
+#if PLATFORM(WIN) || (PLATFORM(QT) && OS(WINDOWS))
 #undef NO_ERROR
 #elif PLATFORM(GTK)
 // This define is from the X11 headers, but it's used below, so we must undefine it.
 #undef VERSION
 #endif
 
-#if PLATFORM(MAC) || PLATFORM(GTK) || PLATFORM(QT) || PLATFORM(EFL) || PLATFORM(BLACKBERRY)
+#if PLATFORM(MAC) || PLATFORM(GTK) || PLATFORM(QT) || PLATFORM(EFL) || PLATFORM(BLACKBERRY) || PLATFORM(WIN)
 #include "ANGLEWebKitBridge.h"
 #endif
 
 #if PLATFORM(MAC)
-#include <OpenGL/OpenGL.h>
 #include <wtf/RetainPtr.h>
 OBJC_CLASS CALayer;
 OBJC_CLASS WebGLLayer;
@@ -56,8 +58,6 @@ OBJC_CLASS WebGLLayer;
 QT_BEGIN_NAMESPACE
 class QPainter;
 class QRect;
-class QGLWidget;
-class QGLContext;
 class QOpenGLContext;
 class QSurface;
 QT_END_NAMESPACE
@@ -66,6 +66,8 @@ typedef unsigned int GLuint;
 #endif
 
 #if PLATFORM(MAC)
+typedef struct _CGLContextObject *CGLContextObj;
+
 typedef CGLContextObj PlatformGraphicsContext3D;
 #elif PLATFORM(QT)
 typedef QOpenGLContext* PlatformGraphicsContext3D;
@@ -75,18 +77,10 @@ typedef void* PlatformGraphicsContext3D;
 typedef void* PlatformGraphicsSurface3D;
 #endif
 
-#if (PLATFORM(CHROMIUM) || PLATFORM(BLACKBERRY)) && USE(SKIA)
-class GrContext;
-#endif
-
 // These are currently the same among all implementations.
 const PlatformGraphicsContext3D NullPlatformGraphicsContext3D = 0;
 const Platform3DObject NullPlatform3DObject = 0;
 
-#if USE(CG)
-#include <CoreGraphics/CGContext.h>
-#endif
-
 namespace WebCore {
 class DrawingBuffer;
 class Extensions3D;
@@ -101,6 +95,7 @@ class Extensions3DQt;
 class HostWindow;
 class Image;
 class ImageBuffer;
+class ImageSource;
 class ImageData;
 class IntRect;
 class IntSize;
@@ -246,6 +241,7 @@ public:
         INT = 0x1404,
         UNSIGNED_INT = 0x1405,
         FLOAT = 0x1406,
+        HALF_FLOAT_OES = 0x8D61,
         FIXED = 0x140C,
         DEPTH_COMPONENT = 0x1902,
         ALPHA = 0x1906,
@@ -488,43 +484,17 @@ public:
     PlatformGraphicsContext3D platformGraphicsContext3D() const { return m_contextObj; }
     Platform3DObject platformTexture() const { return m_compositorTexture; }
     CALayer* platformLayer() const { return reinterpret_cast<CALayer*>(m_webGLLayer.get()); }
-#elif PLATFORM(CHROMIUM) || PLATFORM(BLACKBERRY)
-    PlatformGraphicsContext3D platformGraphicsContext3D() const;
-    Platform3DObject platformTexture() const;
-#if USE(SKIA)
-    GrContext* grContext();
-#endif
-#if USE(ACCELERATED_COMPOSITING)
-    PlatformLayer* platformLayer() const;
-#endif
-#elif PLATFORM(QT)
-    PlatformGraphicsContext3D platformGraphicsContext3D();
-    Platform3DObject platformTexture() const;
-#if USE(ACCELERATED_COMPOSITING)
-    PlatformLayer* platformLayer() const;
-#endif
-#elif PLATFORM(GTK)
-    PlatformGraphicsContext3D platformGraphicsContext3D();
-    Platform3DObject platformTexture() const { return m_texture; }
-#if USE(ACCELERATED_COMPOSITING)
-    PlatformLayer* platformLayer() const;
-#endif
-#elif PLATFORM(EFL)
-    PlatformGraphicsContext3D platformGraphicsContext3D();
-    Platform3DObject platformTexture() const { return m_texture; }
-#if USE(ACCELERATED_COMPOSITING)
-    PlatformLayer* platformLayer() const;
-#endif
 #else
-    PlatformGraphicsContext3D platformGraphicsContext3D() const { return NullPlatformGraphicsContext3D; }
-    Platform3DObject platformTexture() const { return NullPlatform3DObject; }
-#if USE(ACCELERATED_COMPOSITING)
-    PlatformLayer* platformLayer() const { return 0; }
+    PlatformGraphicsContext3D platformGraphicsContext3D();
+    Platform3DObject platformTexture() const;
+#if USE(ACCELERATED_COMPOSITING) 
+    PlatformLayer* platformLayer() const;
 #endif
 #endif
+
     bool makeContextCurrent();
 
-#if PLATFORM(MAC) || PLATFORM(CHROMIUM) || PLATFORM(GTK) || PLATFORM(QT) || PLATFORM(EFL) || PLATFORM(BLACKBERRY)
+#if PLATFORM(MAC) || PLATFORM(GTK) || PLATFORM(QT) || PLATFORM(EFL) || PLATFORM(BLACKBERRY) || PLATFORM(WIN)
     // With multisampling on, blit from multisampleFBO to regular FBO.
     void prepareTexture();
 #endif
@@ -564,18 +534,6 @@ public:
                                      unsigned int* imageSizeInBytes,
                                      unsigned int* paddingInBytes);
 
-    // Extracts the contents of the given Image into the passed Vector,
-    // packing the pixel data according to the given format and type,
-    // and obeying the flipY, premultiplyAlpha, and ignoreGammaAndColorProfile
-    // flags. Returns true upon success.
-    static bool extractImageData(Image*,
-                          GC3Denum format,
-                          GC3Denum type,
-                          bool flipY,
-                          bool premultiplyAlpha,
-                          bool ignoreGammaAndColorProfile,
-                          Vector<uint8_t>& data);
-
     // Extracts the contents of the given ImageData into the passed Vector,
     // packing the pixel data according to the given format and type,
     // and obeying the flipY and premultiplyAlpha flags. Returns true
@@ -599,55 +557,68 @@ public:
                             const void* pixels,
                             Vector<uint8_t>& data);
 
-    // Flips the given image data vertically, in-place.
-    static void flipVertically(void* imageData,
-                        unsigned int width,
-                        unsigned int height,
-                        unsigned int bytesPerPixel,
-                        unsigned int unpackAlignment);
 
     // Attempt to enumerate all possible native image formats to
     // reduce the amount of temporary allocations during texture
     // uploading. This enum must be public because it is accessed
     // by non-member functions.
-    enum SourceDataFormat {
-        SourceFormatRGBA8 = 0,
-        SourceFormatRGBA16Little,
-        SourceFormatRGBA16Big,
-        SourceFormatRGBA32F,
-        SourceFormatRGB8,
-        SourceFormatRGB16Little,
-        SourceFormatRGB16Big,
-        SourceFormatRGB32F,
-        SourceFormatBGR8,
-        SourceFormatBGRA8,
-        SourceFormatBGRA16Little,
-        SourceFormatBGRA16Big,
-        SourceFormatARGB8,
-        SourceFormatARGB16Little,
-        SourceFormatARGB16Big,
-        SourceFormatABGR8,
-        SourceFormatRGBA5551,
-        SourceFormatRGBA4444,
-        SourceFormatRGB565,
-        SourceFormatR8,
-        SourceFormatR16Little,
-        SourceFormatR16Big,
-        SourceFormatR32F,
-        SourceFormatRA8,
-        SourceFormatRA16Little,
-        SourceFormatRA16Big,
-        SourceFormatRA32F,
-        SourceFormatAR8,
-        SourceFormatAR16Little,
-        SourceFormatAR16Big,
-        SourceFormatA8,
-        SourceFormatA16Little,
-        SourceFormatA16Big,
-        SourceFormatA32F,
-        SourceFormatNumFormats
+    enum DataFormat {
+        DataFormatRGBA8 = 0,
+        DataFormatRGBA16Little,
+        DataFormatRGBA16Big,
+        DataFormatRGBA32F,
+        DataFormatRGB8,
+        DataFormatRGB16Little,
+        DataFormatRGB16Big,
+        DataFormatRGB32F,
+        DataFormatBGR8,
+        DataFormatBGRA8,
+        DataFormatBGRA16Little,
+        DataFormatBGRA16Big,
+        DataFormatARGB8,
+        DataFormatARGB16Little,
+        DataFormatARGB16Big,
+        DataFormatABGR8,
+        DataFormatRGBA5551,
+        DataFormatRGBA4444,
+        DataFormatRGB565,
+        DataFormatR8,
+        DataFormatR16Little,
+        DataFormatR16Big,
+        DataFormatR32F,
+        DataFormatRA8,
+        DataFormatRA16Little,
+        DataFormatRA16Big,
+        DataFormatRA32F,
+        DataFormatAR8,
+        DataFormatAR16Little,
+        DataFormatAR16Big,
+        DataFormatA8,
+        DataFormatA16Little,
+        DataFormatA16Big,
+        DataFormatA32F,
+        DataFormatNumFormats
     };
 
+    // Check if the format is one of the formats from the ImageData or DOM elements.
+    // The formats from ImageData is always RGBA8.
+    // The formats from DOM elements vary with Graphics ports. It can only be RGBA8 or BGRA8 for non-CG port while a little more for CG port.
+    static ALWAYS_INLINE bool srcFormatComeFromDOMElementOrImageData(DataFormat SrcFormat)
+    {
+#if USE(CG)
+#if CPU(BIG_ENDIAN)
+    return SrcFormat == DataFormatRGBA8 || SrcFormat == DataFormatARGB8 || SrcFormat == DataFormatRGB8;
+#else
+    // That LITTLE_ENDIAN case has more possible formats than BIG_ENDIAN case is because some decoded image data is actually big endian
+    // even on little endian architectures.
+    return SrcFormat == DataFormatBGRA8 || SrcFormat == DataFormatABGR8 || SrcFormat == DataFormatBGR8
+        || SrcFormat == DataFormatRGBA8 || SrcFormat == DataFormatARGB8 || SrcFormat == DataFormatRGB8;
+#endif
+#else
+    return SrcFormat == DataFormatBGRA8 || SrcFormat == DataFormatRGBA8;
+#endif
+    }
+
     //----------------------------------------------------------------------
     // Entry points for WebGL.
     //
@@ -799,16 +770,13 @@ public:
 
     void reshape(int width, int height);
 
-#if USE(CG)
-    static void paintToCanvas(const unsigned char* imagePixels, int imageWidth, int imageHeight,
-                              int canvasWidth, int canvasHeight, CGContextRef);
-#elif PLATFORM(GTK) || PLATFORM(EFL)
+#if PLATFORM(GTK) || PLATFORM(EFL) || USE(CAIRO)
     void paintToCanvas(const unsigned char* imagePixels, int imageWidth, int imageHeight,
                        int canvasWidth, int canvasHeight, PlatformContextCairo* context);
 #elif PLATFORM(QT)
     void paintToCanvas(const unsigned char* imagePixels, int imageWidth, int imageHeight,
                        int canvasWidth, int canvasHeight, QPainter* context);
-#elif PLATFORM(BLACKBERRY)
+#elif PLATFORM(BLACKBERRY) || USE(CG)
     void paintToCanvas(const unsigned char* imagePixels, int imageWidth, int imageHeight,
                        int canvasWidth, int canvasHeight, GraphicsContext*);
 #endif
@@ -874,35 +842,6 @@ public:
 
     static unsigned getChannelBitsByFormat(GC3Denum);
 
-  private:
-    GraphicsContext3D(Attributes, HostWindow*, RenderStyle = RenderOffscreen);
-
-    // Each platform must provide an implementation of this method.
-    //
-    // Gets the data for the given Image into outputVector in the
-    // format specified by the (OpenGL-style) format and type
-    // arguments. Despite the fact that the outputVector contains
-    // uint8_t, if the format and type specify packed pixels, then
-    // it will essentially contain uint16_t after the extraction
-    // process.
-    //
-    // If premultiplyAlpha is true, the alpha channel, if any,
-    // will be multiplied into the color channels during the
-    // extraction process. This premultiplication occurs before
-    // any packing of pixel data.
-    //
-    // If ignoreGammaAndColorProfile is true, gamma correction and ICC
-    // profile won't be applied.
-    //
-    // No vertical flip of the image data is performed by this
-    // method.
-    static bool getImageData(Image*,
-                      GC3Denum format,
-                      GC3Denum type,
-                      bool premultiplyAlpha,
-                      bool ignoreGammaAndColorProfile,
-                      Vector<uint8_t>& outputVector);
-
     // Possible alpha operations that may need to occur during
     // pixel packing. FIXME: kAlphaDoUnmultiply is lossy and must
     // be removed.
@@ -912,22 +851,75 @@ public:
         AlphaDoUnmultiply = 2
     };
 
-    // Helper for getImageData which implements packing of pixel
+    enum ImageHtmlDomSource {
+        HtmlDomImage = 0,
+        HtmlDomCanvas = 1,
+        HtmlDomVideo = 2,
+        HtmlDomNone = 3
+    };
+
+    // Packs the contents of the given Image which is passed in |pixels| into the passed Vector
+    // according to the given format and type, and obeying the flipY and AlphaOp flags.
+    // Returns true upon success.
+    static bool packImageData(Image*, const void* pixels, GC3Denum format, GC3Denum type, bool flipY, AlphaOp, DataFormat sourceFormat, unsigned width, unsigned height, unsigned sourceUnpackAlignment, Vector<uint8_t>& data);
+
+    class ImageExtractor {
+    public:
+        ImageExtractor(Image*, ImageHtmlDomSource, bool premultiplyAlpha, bool ignoreGammaAndColorProfile);
+
+        // Each platform must provide an implementation of this method to deallocate or release resources
+        // associated with the image if needed.
+        ~ImageExtractor();
+
+        bool extractSucceeded() { return m_extractSucceeded; }
+        const void* imagePixelData() { return m_imagePixelData; }
+        unsigned imageWidth() { return m_imageWidth; }
+        unsigned imageHeight() { return m_imageHeight; }
+        DataFormat imageSourceFormat() { return m_imageSourceFormat; }
+        AlphaOp imageAlphaOp() { return m_alphaOp; }
+        unsigned imageSourceUnpackAlignment() { return m_imageSourceUnpackAlignment; }
+        ImageHtmlDomSource imageHtmlDomSource() { return m_imageHtmlDomSource; }
+    private:
+        // Each platform must provide an implementation of this method.
+        // Extracts the image and keeps track of its status, such as width, height, Source Alignment, format and AlphaOp etc,
+        // needs to lock the resources or relevant data if needed and returns true upon success
+        bool extractImage(bool premultiplyAlpha, bool ignoreGammaAndColorProfile);
+
+#if USE(CAIRO)
+        ImageSource* m_decoder;
+        RefPtr<cairo_surface_t> m_imageSurface;
+#elif USE(CG)
+        CGImageRef m_cgImage;
+        RetainPtr<CGImageRef> m_decodedImage;
+        RetainPtr<CFDataRef> m_pixelData;
+        OwnArrayPtr<uint8_t> m_formalizedRGBA8Data;
+#elif PLATFORM(QT)
+        QImage m_qtImage;
+#elif PLATFORM(BLACKBERRY)
+        Vector<unsigned> m_imageData;
+#endif
+        Image* m_image;
+        ImageHtmlDomSource m_imageHtmlDomSource;
+        bool m_extractSucceeded;
+        const void* m_imagePixelData;
+        unsigned m_imageWidth;
+        unsigned m_imageHeight;
+        DataFormat m_imageSourceFormat;
+        AlphaOp m_alphaOp;
+        unsigned m_imageSourceUnpackAlignment;
+    };
+
+private:
+    GraphicsContext3D(Attributes, HostWindow*, RenderStyle = RenderOffscreen);
+
+    // Helper for packImageData/extractImageData/extractTextureData which implement packing of pixel
     // data into the specified OpenGL destination format and type.
     // A sourceUnpackAlignment of zero indicates that the source
     // data is tightly packed. Non-zero values may take a slow path.
     // Destination data will have no gaps between rows.
-    static bool packPixels(const uint8_t* sourceData,
-                    SourceDataFormat sourceDataFormat,
-                    unsigned int width,
-                    unsigned int height,
-                    unsigned int sourceUnpackAlignment,
-                    unsigned int destinationFormat,
-                    unsigned int destinationType,
-                    AlphaOp alphaOp,
-                    void* destinationData);
+    static bool packPixels(const uint8_t* sourceData, DataFormat sourceDataFormat, unsigned width, unsigned height, unsigned sourceUnpackAlignment, unsigned destinationFormat, unsigned destinationType, AlphaOp, void* destinationData, bool flipY);
 
-#if PLATFORM(MAC) || PLATFORM(GTK) || PLATFORM(QT) || PLATFORM(EFL) || PLATFORM(BLACKBERRY)
+#if PLATFORM(MAC) || PLATFORM(GTK) || PLATFORM(QT) || PLATFORM(EFL) || PLATFORM(BLACKBERRY) || PLATFORM(WIN)
     // Take into account the user's requested context creation attributes,
     // in particular stencil and antialias, and determine which could or
     // could not be honored based on the capabilities of the OpenGL
@@ -965,7 +957,7 @@ public:
     void* m_context;
 #endif
 
-#if PLATFORM(MAC) || PLATFORM(GTK) || PLATFORM(QT) || PLATFORM(EFL) || PLATFORM(BLACKBERRY)
+#if PLATFORM(MAC) || PLATFORM(GTK) || PLATFORM(QT) || PLATFORM(EFL) || PLATFORM(BLACKBERRY) || PLATFORM(WIN)
     struct SymbolInfo {
         SymbolInfo()
             : type(0)
@@ -1006,7 +998,7 @@ public:
         {
         }
         
-        ShaderSymbolMap& symbolMap(ANGLEShaderSymbolType symbolType)
+        ShaderSymbolMap& symbolMap(enum ANGLEShaderSymbolType symbolType)
         {
             ASSERT(symbolType == SHADER_SYMBOL_TYPE_ATTRIBUTE || symbolType == SHADER_SYMBOL_TYPE_UNIFORM);
             if (symbolType == SHADER_SYMBOL_TYPE_ATTRIBUTE)
@@ -1024,10 +1016,10 @@ public:
     ANGLEWebKitBridge m_compiler;
 #endif
 
-#if PLATFORM(BLACKBERRY) || (PLATFORM(QT) && defined(QT_OPENGL_ES_2)) || (PLATFORM(GTK) && USE(OPENGL_ES_2))
+#if PLATFORM(BLACKBERRY) || (PLATFORM(QT) && defined(QT_OPENGL_ES_2)) || ((PLATFORM(GTK) || PLATFORM(EFL) || PLATFORM(WIN)) && USE(OPENGL_ES_2))
     friend class Extensions3DOpenGLES;
     OwnPtr<Extensions3DOpenGLES> m_extensions;
-#elif !PLATFORM(CHROMIUM)
+#else
     friend class Extensions3DOpenGL;
     OwnPtr<Extensions3DOpenGL> m_extensions;
 #endif
@@ -1052,10 +1044,19 @@ public:
     bool m_layerComposited;
     GC3Duint m_internalColorFormat;
 
-    // For tracking which FBO/texture is bound
-    GC3Duint m_boundFBO;
-    GC3Denum m_activeTexture;
-    GC3Duint m_boundTexture0;
+    struct GraphicsContext3DState {
+        GraphicsContext3DState()
+            : boundFBO(0)
+            , activeTexture(GraphicsContext3D::TEXTURE0)
+            , boundTexture0(0)
+        { }
+
+        GC3Duint boundFBO;
+        GC3Denum activeTexture;
+        GC3Duint boundTexture0;
+    };
+
+    GraphicsContext3DState m_state;
 
     // For multisampling
     GC3Duint m_multisampleFBO;
