@@ -74,14 +74,9 @@ FontPlatformData::FontPlatformData(const FontDescription& description, const Ato
     font.setWeight(toQFontWeight(description.weight()));
     font.setWordSpacing(wordSpacing);
     font.setLetterSpacing(QFont::AbsoluteSpacing, letterSpacing);
-#if QT_VERSION < QT_VERSION_CHECK(5, 1, 0)
-    // To maintain stable baselines for Qt 5.0, keep force integer metrics enabled and ignore font-smoothing setting.
-    font.setStyleStrategy(QFont::ForceIntegerMetrics);
-#else
     if (description.fontSmoothing() == NoSmoothing
         || (description.fontSmoothing() == AutoSmoothing && !Font::shouldUseSmoothing()))
         font.setStyleStrategy(QFont::NoAntialias);
-#endif
 
     m_data->bold = font.bold();
     // WebKit allows font size zero but QFont does not. We will return
