@@ -34,6 +34,7 @@
 #include "Chrome.h"
 #include "ChromeClient.h"
 #include "Color.h"
+#include "ExceptionCodePlaceholder.h"
 #include "FileList.h"
 #include "Font.h"
 #include "FontSelector.h"
@@ -255,12 +256,12 @@ Color RenderThemeQt::platformFocusRingColor() const
     return colorPalette().brush(QPalette::Active, QPalette::Highlight).color();
 }
 
-void RenderThemeQt::systemFont(int, FontDescription&) const
+void RenderThemeQt::systemFont(CSSValueID, FontDescription&) const
 {
     // no-op
 }
 
-Color RenderThemeQt::systemColor(int cssValueId) const
+Color RenderThemeQt::systemColor(CSSValueID cssValueId) const
 {
     QPalette pal = colorPalette();
     switch (cssValueId) {
@@ -314,7 +315,6 @@ void RenderThemeQt::adjustTextFieldStyle(StyleResolver*, RenderStyle* style, Ele
     // padding. Just worth keeping in mind!
     style->setBackgroundColor(Color::transparent);
     style->resetBorder();
-    style->resetPadding();
     computeSizeBasedOnStyle(style);
 }
 
@@ -538,7 +538,7 @@ String RenderThemeQt::extraMediaControlsStyleSheet()
 {
     String result = String(mediaControlsQtUserAgentStyleSheet, sizeof(mediaControlsQtUserAgentStyleSheet));
 
-    if (m_page && m_page->chrome()->requiresFullscreenForVideoPlayback())
+    if (m_page && m_page->chrome().requiresFullscreenForVideoPlayback())
         result.append(String(mediaControlsQtFullscreenUserAgentStyleSheet, sizeof(mediaControlsQtFullscreenUserAgentStyleSheet)));
 
     return result;
@@ -593,7 +593,7 @@ QColor RenderThemeQt::getMediaControlForegroundColor(RenderObject* o) const
     if (!o)
         return fgColor;
 
-    if (o->node()->active())
+    if (o->node() && o->node()->isElementNode() && toElement(o->node())->active())
         fgColor = fgColor.lighter();
 
     if (!mediaElementCanPlay(o))
@@ -677,6 +677,37 @@ bool RenderThemeQt::paintMediaPlayButton(RenderObject* o, const PaintInfo& paint
     return false;
 }
 
+bool RenderThemeQt::paintMediaToggleClosedCaptionsButton(RenderObject* o, const PaintInfo& paintInfo, const IntRect& r)
+{
+    HTMLMediaElement* mediaElement = toParentMediaElement(o);
+    if (!mediaElement)
+        return false;
+
+    QSharedPointer<StylePainter> p = getStylePainter(paintInfo);
+    if (p.isNull() || !p->isValid())
+        return true;
+
+    p->painter->setRenderHint(QPainter::Antialiasing, true);
+
+    paintMediaBackground(p->painter, r);
+
+    WorldMatrixTransformer transformer(p->painter, o, r);
+    p->painter->setBrush(getMediaControlForegroundColor(o));
+
+    QPainterPath captionBubble;
+    captionBubble.moveTo(98.766, 43.244);
+    captionBubble.cubicTo(captionBubble.currentPosition() + QPointF(0, -23.163), captionBubble.currentPosition() + QPointF(-21.775, -41.94), captionBubble.currentPosition() + QPointF(-48.637, -41.94));
+    captionBubble.cubicTo(captionBubble.currentPosition() + QPointF(-26.859, 0), captionBubble.currentPosition() + QPointF(-48.635, 18.777), captionBubble.currentPosition() + QPointF(-48.635, 41.94));
+    captionBubble.cubicTo(captionBubble.currentPosition() + QPointF(0, 18.266), captionBubble.currentPosition() + QPointF(13.546, 33.796), captionBubble.currentPosition() + QPointF(32.444, 39.549));
+    captionBubble.cubicTo(captionBubble.currentPosition() + QPointF(1.131, 8.356), captionBubble.currentPosition() + QPointF(26.037, 24.255), captionBubble.currentPosition() + QPointF(22.864, 19.921));
+    captionBubble.cubicTo(captionBubble.currentPosition() + QPointF(-4.462, -6.096), captionBubble.currentPosition() + QPointF(-5.159, -13.183), captionBubble.currentPosition() + QPointF(-5.07, -17.566));
+    captionBubble.cubicTo(QPointF(77.85, 84.397), QPointF(98.766, 65.923), QPointF(98.766, 43.224));
+    captionBubble.closeSubpath();
+
+    p->painter->drawPath(captionBubble);
+    return false;
+}
+
 bool RenderThemeQt::paintMediaSeekBackButton(RenderObject*, const PaintInfo&, const IntRect&)
 {
     // We don't want to paint this at the moment.
@@ -740,10 +771,10 @@ bool RenderThemeQt::paintMediaVolumeSliderTrack(RenderObject *o, const PaintInfo
     p->painter->setBrush(scaleColor);
     p->painter->drawRect(left, top, width, height);
 
-    if (!o->node() || !o->node()->hasTagName(inputTag))
+    if (!o->node() || !isHTMLInputElement(o->node()))
         return false;
 
-    HTMLInputElement* slider = static_cast<HTMLInputElement*>(o->node());
+    HTMLInputElement* slider = toHTMLInputElement(o->node());
 
     // Position the inner rectangle
     height = height * slider->valueAsNumber();
@@ -790,10 +821,9 @@ bool RenderThemeQt::paintMediaSliderTrack(RenderObject* o, const PaintInfo& pain
             p->painter->setBrush(getMediaControlForegroundColor());
 
             // Paint each buffered section
-            ExceptionCode ex;
             for (int i = 0; i < buffered->length(); i++) {
-                float startX = (buffered->start(i, ex) / player->duration()) * 100;
-                float width = ((buffered->end(i, ex) / player->duration()) * 100) - startX;
+                float startX = (buffered->start(i, IGNORE_EXCEPTION) / player->duration()) * 100;
+                float width = ((buffered->end(i, IGNORE_EXCEPTION) / player->duration()) * 100) - startX;
                 p->painter->drawRect(startX, 37, width, 26);
             }
         }
@@ -873,22 +903,10 @@ String RenderThemeQt::fileListNameForWidth(const FileList* fileList, const Font&
     return string;
 }
 
-StylePainter::StylePainter(RenderThemeQt* theme, const PaintInfo& paintInfo)
-    : painter(0)
+StylePainter::StylePainter(GraphicsContext* context)
+    : painter(context->platformContext())
 {
-    Q_UNUSED(theme);
-    ASSERT(paintInfo.context);
-    init(paintInfo.context);
-}
-
-StylePainter::StylePainter()
-    : painter(0)
-{
-}
-
-void StylePainter::init(GraphicsContext* context)
-{
-    painter = static_cast<QPainter*>(context->platformContext());
+    ASSERT(context);
 
     if (painter) {
         // the styles often assume being called with a pristine painter where no brush is set,
