@@ -209,7 +209,7 @@ QPalette RenderThemeQtMobile::colorPalette() const
 }
 
 StylePainterMobile::StylePainterMobile(RenderThemeQtMobile* theme, const PaintInfo& paintInfo)
-    : StylePainter(theme, paintInfo)
+    : StylePainter(paintInfo.context)
 {
     m_previousSmoothPixmapTransform = painter->testRenderHint(QPainter::SmoothPixmapTransform);
     if (!m_previousSmoothPixmapTransform)
@@ -656,6 +656,7 @@ void RenderThemeQtMobile::computeSizeBasedOnStyle(RenderStyle* renderStyle) cons
         break;
     }
     default:
+        renderStyle->resetPadding();
         break;
     }
     // If the width and height are both specified, then we have nothing to do.
@@ -866,7 +867,7 @@ bool RenderThemeQtMobile::paintSliderTrack(RenderObject* o, const PaintInfo& pi,
     if (!p.isValid())
         return true;
 
-    HTMLInputElement* slider = static_cast<HTMLInputElement*>(o->node());
+    HTMLInputElement* slider = toHTMLInputElement(o->node());
 
     const double min = slider->minimum();
     const double max = slider->maximum();
@@ -901,7 +902,7 @@ bool RenderThemeQtMobile::paintSliderThumb(RenderObject* o, const PaintInfo& pi,
 
 bool RenderThemeQtMobile::checkMultiple(RenderObject* o) const
 {
-    HTMLSelectElement* select = o ? static_cast<HTMLSelectElement*>(o->node()) : 0;
+    HTMLSelectElement* select = o ? toHTMLSelectElement(o->node()) : 0;
     return select ? select->multiple() : false;
 }
 
