@@ -28,6 +28,7 @@
 #include <QtCore/qshareddata.h>
 
 namespace WebCore {
+    class GroupSettings;
     class Settings;
 }
 
@@ -38,6 +39,8 @@ QT_BEGIN_NAMESPACE
 class QUrl;
 QT_END_NAMESPACE
 
+QWEBKIT_EXPORT void qt_networkAccessAllowed(bool isAllowed);
+
 class QWEBKIT_EXPORT QWebSettings {
 public:
     enum FontFamily {
@@ -82,7 +85,8 @@ public:
         CSSGridLayoutEnabled,
         ScrollAnimatorEnabled,
         CaretBrowsingEnabled,
-        NotificationsEnabled
+        NotificationsEnabled,
+        WebAudioEnabled
     };
     enum WebGraphic {
         MissingImageGraphic,
@@ -173,7 +177,7 @@ private:
     Q_DISABLE_COPY(QWebSettings)
 
     QWebSettings();
-    QWebSettings(WebCore::Settings *settings);
+    QWebSettings(WebCore::Settings *settings, WebCore::GroupSettings *groupSettings);
     ~QWebSettings();
 
     QWebSettingsPrivate *d;
