@@ -32,24 +32,20 @@ QT_END_NAMESPACE
 namespace WebCore {
 
 class ChromeClientQt;
-class FullScreenVideoWidget;
 class HTMLVideoElement;
 class Node;
 #if USE(QT_MULTIMEDIA)
 class MediaPlayerPrivateQt;
 #endif
-#if USE(QTKIT)
-class QTKitFullScreenVideoHandler;
-#endif
 
 // We do not use ENABLE or USE because moc does not expand these macros.
-#if defined(WTF_USE_GSTREAMER) && WTF_USE_GSTREAMER && !defined(GST_API_VERSION_1)
+#if defined(WTF_USE_GSTREAMER) && WTF_USE_GSTREAMER && defined(WTF_USE_NATIVE_FULLSCREEN_VIDEO) && WTF_USE_NATIVE_FULLSCREEN_VIDEO
 class FullScreenVideoWindow;
 
 class GStreamerFullScreenVideoHandler : public QObject {
     Q_OBJECT
 public:
-    GStreamerFullScreenVideoHandler();
+    GStreamerFullScreenVideoHandler(ChromeClientQt*);
     ~GStreamerFullScreenVideoHandler() { }
     void setVideoElement(HTMLVideoElement*);
 
@@ -62,6 +58,7 @@ public Q_SLOTS:
 private:
     HTMLVideoElement* m_videoElement;
     FullScreenVideoWindow* m_fullScreenWidget;
+    ChromeClientQt* m_chromeClient;
 };
 #endif
 
@@ -90,12 +87,9 @@ private:
 #if USE(QT_MULTIMEDIA)
     QWebFullScreenVideoHandler* m_FullScreenVideoHandler;
 #endif
-#if USE(GSTREAMER) && !defined(GST_API_VERSION_1)
+#if USE(GSTREAMER) && USE(NATIVE_FULLSCREEN_VIDEO)
     GStreamerFullScreenVideoHandler* m_FullScreenVideoHandlerGStreamer;
 #endif
-#if USE(QTKIT)
-    QTKitFullScreenVideoHandler* m_FullScreenVideoHandlerQTKit;
-#endif
 };
 
 }
