@@ -20,6 +20,7 @@
 #ifndef QWebPageAdapter_h
 #define QWebPageAdapter_h
 
+#include "PlatformExportMacros.h"
 #include "QWebPageClient.h"
 #include "ViewportArguments.h"
 #include "qwebhistory.h"
@@ -46,6 +47,8 @@ QT_END_NAMESPACE
 
 namespace WebCore {
 class ChromeClientQt;
+class DeviceOrientationClient;
+class DeviceMotionClient;
 class GeolocationClientQt;
 class Page;
 class UndoStep;
@@ -90,7 +93,14 @@ public:
     F(ToggleBold, WebCore::ContextMenuItemTagBold) SEPARATOR \
     F(ToggleItalic, WebCore::ContextMenuItemTagItalic) SEPARATOR \
     F(ToggleUnderline, WebCore::ContextMenuItemTagUnderline) SEPARATOR \
-    F(SelectAll, WebCore::ContextMenuItemTagSelectAll)
+    F(SelectAll, WebCore::ContextMenuItemTagSelectAll) SEPARATOR \
+    F(DownloadMediaToDisk, WebCore::ContextMenuItemTagDownloadMediaToDisk) SEPARATOR \
+    F(CopyMediaUrlToClipboard, WebCore::ContextMenuItemTagCopyMediaLinkToClipboard) SEPARATOR \
+    F(ToggleMediaControls, WebCore::ContextMenuItemTagToggleMediaControls) SEPARATOR \
+    F(ToggleMediaLoop, WebCore::ContextMenuItemTagToggleMediaLoop) SEPARATOR \
+    F(ToggleMediaPlayPause, WebCore::ContextMenuItemTagMediaPlayPause) SEPARATOR \
+    F(ToggleMediaMute, WebCore::ContextMenuItemTagMediaMute) SEPARATOR \
+    F(ToggleVideoFullscreen, WebCore::ContextMenuItemTagToggleVideoFullscreen)
 #define COMMA_SEPARATOR ,
 #define SEMICOLON_SEPARATOR ;
 #define DEFINE_ACTION(Name, Value) \
@@ -110,7 +120,10 @@ public:
         FindBackward = 1,
         FindCaseSensitively = 2,
         FindWrapsAroundDocument = 4,
-        HighlightAllOccurrences = 8
+        HighlightAllOccurrences = 8,
+        FindAtWordBeginningsOnly = 16,
+        TreatMedialCapitalAsWordBeginning = 32,
+        FindBeginsInSelection = 64
     };
 
     // valid values matching those from ScrollTypes.h
@@ -129,6 +142,14 @@ public:
         ScrollByDocument
     };
 
+    // Must match with values of QWebPage::VisibilityState enum.
+    enum VisibilityState {
+        VisibilityStateVisible,
+        VisibilityStateHidden,
+        VisibilityStatePrerender,
+        VisibilityStateUnloaded
+    };
+
     QWebPageAdapter();
     virtual ~QWebPageAdapter();
 
@@ -238,11 +259,14 @@ public:
         Q_DECLARE_FLAGS(Traits, Trait);
         Traits traits;
         QList<MenuItemDescription> subMenu;
-        QString subMenuTitle;
+        QString title;
     };
     virtual void createAndSetCurrentContextMenu(const QList<MenuItemDescription>&, QBitArray*) = 0;
     virtual bool handleScrollbarContextMenuEvent(QContextMenuEvent*, bool, ScrollDirection*, ScrollGranularity*) = 0;
 
+    void setVisibilityState(VisibilityState);
+    VisibilityState visibilityState() const;
+
     static QWebPageAdapter* kit(WebCore::Page*);
     WebCore::ViewportArguments viewportArguments() const;
     void registerUndoStep(WTF::PassRefPtr<WebCore::UndoStep>);
@@ -259,19 +283,6 @@ public:
 
     bool findText(const QString& subString, FindFlag options);
 
-    class TouchAdjuster {
-    public:
-        TouchAdjuster(unsigned topPadding, unsigned rightPadding, unsigned bottomPadding, unsigned leftPadding);
-
-        WebCore::IntPoint findCandidatePointForTouch(const WebCore::IntPoint&, WebCore::Document*) const;
-
-    private:
-        unsigned m_topPadding;
-        unsigned m_rightPadding;
-        unsigned m_bottomPadding;
-        unsigned m_leftPadding;
-    };
-
     void adjustPointForClicking(QMouseEvent*);
 
     void mouseMoveEvent(QMouseEvent*);
@@ -307,7 +318,7 @@ public:
     QString contextMenuItemTagForAction(MenuAction, bool* checkable) const;
 
     QStringList supportedContentTypes() const;
-#if ENABLE(GEOLOCATION)
+#if ENABLE(GEOLOCATION) && HAVE(QTPOSITIONING)
     void setGeolocationEnabledForFrame(QWebFrameAdapter*, bool);
 #endif
 #if ENABLE(NOTIFICATIONS) || ENABLE(LEGACY_NOTIFICATIONS)
@@ -343,6 +354,7 @@ public:
     };
 
     ViewportAttributes viewportAttributesForSize(const QSize& availableSize, const QSize& deviceSize) const;
+    void setDevicePixelRatio(float devicePixelRatio);
 
     QWebSettings *settings;
 
@@ -357,12 +369,15 @@ public:
     QBasicTimer tripleClickTimer;
 
     bool clickCausedFocus;
+    bool m_useNativeVirtualKeyAsDOMKey;
     quint64 m_totalBytes;
     quint64 m_bytesReceived;
     QWebHistory history;
 
 private:
     QNetworkAccessManager *networkManager;
+    WebCore::DeviceOrientationClient* m_deviceOrientationClient;
+    WebCore::DeviceMotionClient* m_deviceMotionClient;
 
 public:
     static bool drtRun;
